@@ -793,9 +793,6 @@ header_parse (GMimeParser *parser, struct _header_raw **tail)
 	char *start;
 	size_t hlen;
 	
-	header = g_new (struct _header_raw, 1);
-	header->next = NULL;
-	
 	*priv->headerptr = '\0';
 	inptr = priv->headerbuf;
 	while (*inptr && *inptr != ':' && !is_type (*inptr, IS_SPACE | IS_CTRL))
@@ -814,6 +811,10 @@ header_parse (GMimeParser *parser, struct _header_raw **tail)
 	}
 	
 	hlen = inptr - priv->headerbuf;
+	
+	header = g_new (struct _header_raw, 1);
+	header->next = NULL;
+	
 	header->name = g_strndup (priv->headerbuf, hlen);
 	
 	/* skip over leading lwsp */
@@ -850,7 +851,7 @@ static int
 parser_step_headers (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	gboolean valid = TRUE, fieldname = TRUE;
+	gboolean eoln, valid = TRUE, fieldname = TRUE;
 	struct _header_raw *hend;
 	register char *inptr;
 	char *start, *inend;
@@ -882,10 +883,13 @@ parser_step_headers (GMimeParser *parser)
 		while (inptr < inend) {
 			start = inptr;
 			
-			if (fieldname && *inptr != '\n') {
+			eoln = inptr[0] == '\n' || (inptr[0] == '\r' && inptr[1] == '\n');
+			if (fieldname && !eoln) {
 				/* scan and validate the field name */
 				if (*inptr != ':') {
+					/* Note: see optimization comment [1] */
 					*inend = ':';
+					
 					while (*inptr != ':') {
 						if (is_type (*inptr, IS_SPACE | IS_CTRL)) {
 							valid = FALSE;
@@ -902,6 +906,7 @@ parser_step_headers (GMimeParser *parser)
 						goto refill;
 					}
 					
+					/* Note: see optimization comment [1] */
 					*inend = '\n';
 				} else if (*inptr == ':') {
 					valid = FALSE;
@@ -940,13 +945,13 @@ parser_step_headers (GMimeParser *parser)
 				goto refill;
 			}
 			
-			/* check to see if we've reached the end of the headers */
-			if (!priv->midline && (inptr == start || (len == 1 && *start == '\r')))
-				goto headers_end;
-			
 			if (inptr > start && inptr[-1] == '\r')
 				len--;
 			
+			/* check to see if we've reached the end of the headers */
+			if (!priv->midline && len == 0)
+				goto headers_end;
+			
 			header_append (priv, start, len);
 			
 			/* inptr has to be less than inend - 1 */
@@ -1269,8 +1274,10 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 		/* last '\n' belongs to the boundary */
 		if (priv->persist_stream && priv->seekable)
 			end = parser_offset (priv, NULL) - crlf;
+		else if (content->len > crlf)
+			g_byte_array_set_size (content, content->len - crlf);
 		else
-			g_byte_array_set_size (content, MAX (content->len - crlf, 0));
+			g_byte_array_set_size (content, 0);
 	} else if (priv->persist_stream && priv->seekable) {
 		end = parser_offset (priv, NULL);
 	}
@@ -1590,7 +1597,7 @@ static GMimeMessage *
 parser_construct_message (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	unsigned int content_length = ULONG_MAX;
+	unsigned long content_length = ULONG_MAX;
 	GMimeContentType *content_type;
 	struct _header_raw *header;
 	GMimeMessage *message;
